import { Action } from '@ngrx/store';
import { take, materialize } from 'rxjs/operators';

import { NGRX_STORE_META, StoreMetadata } from './meta-data';
import { NgRxStoreSelect } from '../service';

export function createReducer<State = any>(
  store: | {new (...args: any[]): any} | any
): (state: State, action: Action | any) => State {
  const isInstance = !store.prototype;
  const clazz = isInstance ? store.constructor : store;

  if (!clazz.hasOwnProperty(NGRX_STORE_META)) {
    throw new Error('A reducer can be created from a @Store decorated class only.');
  }

  const instance = isInstance ? store : new store();
  const { initialState, actions, effects } = clazz[NGRX_STORE_META] as StoreMetadata;

  return function(state: any = initialState, action: Action) {
    const actionMeta = actions[action.type];
    if (actionMeta) {
      const result = instance[actionMeta.methodName](state, action);
      if (result === undefined) {
        if (Array.isArray(state)) {
          return [...state];
        } else {
          return { ...state };
        }
      }
      state = result;
    }

    const effectMeta = effects[action.type];
    if (effectMeta) {
      const retVal = instance[effectMeta.methodName](state, action);
      if (retVal) {
        if (retVal.subscribe) {
          retVal.pipe(materialize()).subscribe((res: any) => {
            if (res.value && NgRxStoreSelect.store) {
              NgRxStoreSelect.store.dispatch(res.value);
            }
          });
        } else if (NgRxStoreSelect.store) {
          if (Array.isArray(retVal)) {
            retVal.forEach(r => NgRxStoreSelect.store && NgRxStoreSelect.store.dispatch(r));
          } else {
            NgRxStoreSelect.store.dispatch(retVal);
          }
        }
      }
    }

    return state;
  };
}
