import {
  ActionType,
  ensureStoreMetadata,
} from '../core';

export function Action(...actionsClasses: ActionType[]) {
  return function(target: any, name: string, descriptor: TypedPropertyDescriptor<any>) {
    const meta = ensureStoreMetadata(target.constructor);

    for (const clazz of actionsClasses) {
      const inst = new clazz();
      const type = inst.type;

      if (meta.actions[type]) {
        throw new Error(
          `@Action for '${type}' is defined multiple times in functions '${meta.actions[type].methodName}' and '${name}'`
        );
      }

      meta.actions[type] = {
        action: clazz,
        methodName: name,
        type
      };
    }
  };
}
