import {
  ActionType,
  ensureStoreMetadata,
} from '../core';

export function Effect(...effectClasses: ActionType[]) {
  return function(target: any, name: string, descriptor: TypedPropertyDescriptor<any>) {
    const meta = ensureStoreMetadata(target.constructor);

    for (const clazz of effectClasses) {
      const inst = new clazz();
      const type = inst.type;

      if (meta.effects[type]) {
        throw new Error(
          `@Effect for '${type}' is defined multiple times in functions '${meta.effects[type].methodName}' and '${name}'`
        );
      }

      meta.effects[type] = {
        action: clazz,
        methodName: name,
        type
      };
    }
  };
}
